import time

from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait


class GbSeleniumDriver:
    """
    Selenium webdriver for Chrome to interact with Goodbudget website

    Attributes:
        webdriver_path (str): path to the chrome webdriver
        use_gui (bool): show browser if True
        screenshot (bool): save screenshot if True

    Methods:
        log_in(in_username, in_password): log into goodbudget
        enter_transaction(
            in_date,
            in_payee,
            in_amount,
            in_envelope,
            in_notes
        ): enter a transaction into goodbudget
        exit_driver(): quits the webdriver
    """

    def __init__(self, webdriver_path, use_gui, screenshot):
        """Sets class attributes and initializes driver."""
        self.webdriver_path = webdriver_path
        self.use_gui = use_gui
        self.screenshot = screenshot
        self.driver = self._initialize_driver()

    def log_in(self, in_username, in_password):
        """Logs into goodbudget."""
        username = self.driver.find_element(By.ID, "username")
        username.send_keys(in_username)

        password = self.driver.find_element(By.ID, "password")
        password.send_keys(in_password)

        login_button = self.driver.find_element(
            By.XPATH,
            "//button[@class='elementor-button elementor-size-sm "
            "elementor-animation-grow']",
        )
        self.driver.execute_script("arguments[0].click();", login_button)
        assert (
            self.driver.title == "Home | Goodbudget"
        ), f"Got browser title of {self.driver.title} instead"
        print("Logged in.\n")

    def enter_transaction(self, in_date, in_payee, in_amount, in_envelope, in_notes):
        """Enters a transaction into goodbudget."""
        print("Entering transaction. Please wait...\n")
        self._click_add_transation()
        self._enter_date(in_date)
        self._enter_payee(in_payee)
        self._enter_amount(in_amount)
        self._enter_envelope(in_envelope)
        self._enter_notes(in_notes)
        self._click_save_transaction()

    def exit_driver(self):
        """Quits the webdriver."""
        if self.screenshot:
            self.driver.save_screenshot("screenshot.png")
            print("\nScreenshot saved to screenshot.png")
        print("\nThank you for using goodbudget_cli! See you next time!")
        self.driver.quit()

    def _initialize_driver(self):
        """Set the initial options and starts the webdriver."""
        # Setup chrome options
        chrome_options = Options()
        if not self.use_gui:
            chrome_options.add_argument("--headless")  # Ensure GUI is off
        chrome_options.add_argument("--no-sandbox")
        chrome_options.add_argument("window-size=1024x768")

        # Set path to chromedriver as per your configuration
        webdriver_service = Service(self.webdriver_path)

        # Choose Chrome Browser
        driver = webdriver.Chrome(service=webdriver_service, options=chrome_options)

        driver.maximize_window()

        driver.get("https://goodbudget.com/login")
        # General check of the login page
        # Extract description from page and print
        description = driver.find_element(By.NAME, "description").get_attribute(
            "content"
        )
        assert (
            description == "Log in to Goodbudget. Budget well. Live life. Do good."
        )  # subject to change
        return driver

    def _click_add_transation(self):
        "Clicks the Add Transaction button."
        add_transaction_button = self.driver.find_element(
            By.LINK_TEXT, "Add Transaction"
        )
        self.driver.execute_script("arguments[0].click();", add_transaction_button)

    def _enter_date(self, in_date):
        "Enters the transaction date in the Date field."
        expense_date = self.driver.find_element(By.ID, "expense-date")
        WebDriverWait(self.driver, 10).until(EC.element_to_be_clickable(expense_date))
        self.driver.execute_script("arguments[0].click();", expense_date)
        expense_date.clear()
        expense_date.send_keys(in_date)

    def _enter_payee(self, in_payee):
        "Enters the transaction payee in the payee field."
        expense_payee = self.driver.find_element(By.ID, "expense-receiver")
        self.driver.execute_script("arguments[0].click();", expense_payee)
        expense_payee.send_keys(in_payee)

    def _enter_amount(self, in_amount):
        "Enters the transaction dollar amount in the Amount field."
        expense_amount = self.driver.find_element(By.ID, "expense-amount")
        self.driver.execute_script("arguments[0].click();", expense_amount)
        expense_amount.send_keys(in_amount)

    def _enter_envelope(self, in_envelope):
        """Chooses the correct envelope from the Envelope dropdown menu."""
        # Could not get Selenium selector to work,
        # so went with solution of typing out the first few letters
        # of the desired envelope.
        actions = ActionChains(self.driver)
        actions.send_keys(
            Keys.TAB
        )  # Big assumption that "Enter Amount" was last field visited
        actions.perform()
        actions.send_keys(in_envelope)
        actions.perform()

    def _enter_notes(self, in_notes):
        "Enters the transaction notes in the Notes field."
        expense_notes = self.driver.find_element(By.ID, "expense-notes")
        self.driver.execute_script("arguments[0].click();", expense_notes)
        expense_notes.send_keys(in_notes)

    def _click_save_transaction(self):
        "Clicks the Save button"
        save_button = self.driver.find_element(By.ID, "addTransactionSave")
        self.driver.execute_script("arguments[0].click();", save_button)

        time.sleep(1)
        print("Success! Your transaction was entered into Goodbudget.\n")



import argparse
import getpass

from .utils.driver import GbSeleniumDriver
from .utils.util import (
    check_config_json,
    format_date,
    get_envelope_from_alias,
    parse_config,
)


def main():
    """Main entry point for running goodbudget_cli."""
    # initialize .config/goodbudget_cli/config.json
    check_config_json()

    # Parser for login email
    parser = argparse.ArgumentParser()
    parser.add_argument("username", help="Username for Goodbudget", action="store")
    parser.add_argument("-g", "--use-gui", help="show the browser", action="store_true")
    parser.add_argument(
        "-s",
        "--screenshot",
        help="save screenshot at the end of the session",
        action="store_true",
    )
    args = parser.parse_args()

    # Get password
    gb_password = getpass.getpass(prompt="Enter your Goodbudget password: ")

    # Log in
    print("Logging in. Please wait...")
    gb_driver = GbSeleniumDriver(
        parse_config()["webdriver_path"], args.use_gui, args.screenshot
    )
    gb_driver.log_in(args.username, gb_password)

    # Allow user to keep adding new transactions until they're done
    more_transactions = True
    while more_transactions:
        input_date = format_date(
            input("Date of transaction (today / yesterday / mm/dd/yyyy): ")
        )
        input_payee = input("Payee: ")
        input_amount = input("Amount: ")
        input_envelope = get_envelope_from_alias()
        input_notes = input("Notes (optional): ")

        summary_of_transaction = f"""
        Summary of your transcation:

            Date: {input_date}
            Payee: {input_payee}
            Amount: ${input_amount}
            Envelope: {input_envelope} (based on your alias of '{input_envelope}')
            Notes: {input_notes if input_notes else "<none>"}
        """
        print(summary_of_transaction)
        input_confirmation = input("Is everything correct? (Y/n) ")
        if input_confirmation.lower() not in ["y", "yes"]:
            print("Whoops. Please try entering your transaction again.\n")
            continue

        gb_driver.enter_transaction(
            input_date, input_payee, input_amount, input_envelope, input_notes
        )

        input_more_transactions = input(
            "Do you want to enter another transaction? (Y/n) "
        )
        if input_more_transactions.lower() not in ["y", "yes"]:
            more_transactions = False

    gb_driver.exit_driver()



import datetime
import json
import os
import pprint
import shutil
from pathlib import Path
from typing import Dict


def parse_config() -> Dict:
    """Parse the config.json, returns as dictionary."""
    config_json = Path("~/.config/goodbudget_cli/config.json").expanduser()
    f = open(config_json)
    data = json.load(f)
    return data


def check_config_json() -> None:
    """Create a config.json in ~/.config/goodbudget_cli if it doesn't exist"""
    config_json = Path("~/.config/goodbudget_cli/config.json").expanduser()
    if not os.path.exists(config_json):
        os.makedirs(config_json.parent, exist_ok=True)
        source = Path(__file__).parent.parent / "config.json"
        shutil.copyfile(source, config_json)
        print(f"WARNING! {config_json} was not found.")
        print("Don't worry, we made one for you.")
        print(
            f"Please go edit {config_json} to set your configuration settings,\n"
            "and then run `gb` again."
        )
        quit()


def format_date(input_date: str) -> str:
    """Format input date as mm/dd/yyyy."""
    if input_date.lower() == "today":
        current_day = datetime.date.today()
        formatted_date = datetime.date.strftime(current_day, "%m/%d/%Y")
    elif input_date.lower() == "yesterday":
        yesterday = datetime.date.today() - datetime.timedelta(days=1)
        formatted_date = datetime.date.strftime(yesterday, "%m/%d/%Y")
    else:
        assert input_date.count("/") == 2, "Format must be mm/dd/yyyy!"
        month, day, year = input_date.split("/")

        assert len(month) in [1, 2]
        if len(month) == 1:
            month = "0" + month
        assert len(day) in [1, 2]
        if len(day) == 1:
            day = "0" + day
        assert len(year) in [2, 4]
        if len(year) == 4:
            assert year.startswith("20")  # assuming it's 20XX
        if len(year) == 2:
            year = "20" + year

        formatted_date = "/".join([month, day, year])

    return formatted_date


def get_envelope_from_alias() -> str:
    """Determine the correct envelope given a alias."""
    envelopes_data = parse_config()["Envelopes"]

    input_phrase = input("Envelope (or type in 'remind'): ")

    if input_phrase.lower() == "remind":
        print("These are your available envelopes and their aliases:\n")
        pprint.pprint(envelopes_data, indent=4)
        print("\n")
        found_envelope = get_envelope_from_alias()
    else:
        # Match alias to envelope
        input_phrase = input_phrase.lower()
        found = False
        for envelope_name, aliases in envelopes_data.items():
            if input_phrase in aliases:
                found_envelope = envelope_name
                found = True
                break
        if not found:
            print(f"Could not determine which envelope {input_phrase} belongs to!")
            print("Please try again.\n")
            found_envelope = get_envelope_from_alias()

    return found_envelope
